/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import type { StyleTuple } from 'css-to-react-native'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import transform from 'css-to-react-native'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

function transformThemeToStyles(theme: ThemeRegistration) {
  const background = theme.colors?.['editor.background'] || '#1e1e1e'
  const foreground = theme.colors?.['editor.foreground'] || '#d4d4d4'

  const styles = StyleSheet.create({
    scrollView: {
      flex: 1,
      minHeight: 20,
    },
    codeContainer: {
      padding: 16,
      minWidth: '100%',
      backgroundColor: background,
    },
    lineContainer: {
      flexDirection: 'row',
      flexWrap: 'wrap',
      backgroundColor: background,
    },
    token: {
      color: foreground,
      fontFamily: monospaceFont,
      backgroundColor: background,
    },
  })

  const tokenStyles: Record<string, TextStyle> = {}
  theme.tokenColors?.forEach((token) => {
    if (!token.settings)
      return
    const style = transform([
      ['color', token.settings.foreground || foreground],
      ['backgroundColor', background],
      ['fontStyle', token.settings.fontStyle || 'normal'],
    ])
    if (token.scope) {
      const scopes = Array.isArray(token.scope) ? token.scope : [token.scope]
      scopes.forEach((scope) => {
        tokenStyles[scope] = style
      })
    }
  })

  return { base: styles, tokens: tokenStyles }
}

interface Props extends SyntaxHighlighterProps {
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const SyntaxHighlighter: React.FC<Props> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId: languageId || String(language),
    theme,
  })

  const styles = useMemo(() => transformThemeToStyles(theme as ThemeRegistration), [theme])

  const cleanStyle = (style: Record<string, any>): TextStyle => {
    const filtered = Object.entries(style)
      .filter(([_key, value]) => value != null)
      .map<StyleTuple>(([key, value]) => [key, value])

    console.log('Cleaning style:', style, 'Filtered:', filtered)
    return transform(filtered) as TextStyle
  }

  const renderToken = (token: ThemedToken, rowIndex: number, tokenIndex: number) => {
    const style = cleanStyle({
      color: token.color,
      fontStyle: token.fontStyle ? 'italic' : 'normal',
      fontWeight: token.fontStyle ? 'bold' : 'normal',
      backgroundColor: theme.colors?.['editor.background'] || '#1e1e1e',
    })

    console.log('Token Style:', {
      input: {
        color: token.color,
        fontStyle: token.fontStyle,
        background: theme.colors?.['editor.background'],
      },
      output: style,
    })

    return (
      <Text key={`${rowIndex}-${tokenIndex}`} style={[styles.base.token, style, { fontSize }, textStyle]}>
        {token.content}
      </Text>
    )
  }

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[styles.base.scrollView, scrollViewProps?.style]}
      contentContainerStyle={[styles.base.codeContainer, scrollViewProps?.contentContainerStyle, containerStyle]}
    >
      <View onStartShouldSetResponder={() => true}>
        {tokens.map((line, rowIndex) => (
          <View key={rowIndex} style={styles.base.lineContainer}>
            {line.map((token, tokenIndex) => renderToken(token, rowIndex, tokenIndex))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
